/*********************************************************************

 ADOBE SYSTEMS INCORPORATED
 Copyright (C) 1994-2001 Adobe Systems Incorporated
 All rights reserved.

 NOTICE: Adobe permits you to use, modify, and distribute this file
 in accordance with the terms of the Adobe license agreement
 accompanying it. If you have received this file from a source other
 than Adobe, then your use, modification, or distribution of it
 requires the prior written permission of Adobe.

 ---------------------------------------------------------------------

 PIMain.c

 - Source code that must be linked into every plug-in.

*********************************************************************/

#include "Environ.h"

#if MAC_PLATFORM

#include <ToolUtils.h>
#include <Resources.h>

short gResFile;
short gLocaleResFile;

#endif	/* MAC_PLATFORM */

#include "PICommon.h"
#include "PIMain.h"
#include "CorCalls.h"

HFT gCoreHFT = 0;
ASUns32 gCoreVersion = 0;

#if DEBUG
void *scratchFourBytes;

/* For special DURING/HANDLER catching */
int gBadReturnCatcher;
#endif

#if PI_ACROSUPPORT_VERSION != 0
HFT gAcroSupportHFT;
ASUns32 gAcroSupportVersion;
#endif

#if PI_COS_VERSION != 0
HFT gCosHFT = 0;
ASUns32 gCosVersion = 0;
#endif

#if PI_PDMODEL_VERSION != 0
HFT gPDModelHFT = 0;
ASUns32 gPDModelVersion = 0;
#endif

#if PI_PDFEDIT_READ_VERSION != 0
HFT gPDFEditReadHFT = 0;
ASUns32 gPDFEditReadVersion = 0;
#endif

#if PI_PDFEDIT_WRITE_VERSION != 0
HFT gPDFEditWriteHFT = 0;
ASUns32 gPDFEditWriteVersion = 0;
#endif

#if PI_PDSYSFONT_VERSION != 0
HFT gPDSysFontHFT = 0;
ASUns32 gPDSysFontVersion = 0;
#endif

#if PI_PAGE_PDE_CONTENT_VERSION != 0
HFT gPagePDEContentHFT = 0;
ASUns32 gPagePDEContentVersion = 0;
#endif

#if PI_ACROVIEW_VERSION != 0
HFT gAcroViewHFT = 0;
ASUns32 gAcroViewVersion = 0;
#endif

#if PI_PDSEDIT_WRITE_VERSION != 0
HFT gPDSWriteHFT = 0;
ASUns32 gPDSWriteVersion = 0;
#endif

#if PI_PDSEDIT_READ_VERSION != 0
HFT gPDSReadHFT = 0;
ASUns32 gPDSReadVersion = 0;
#endif

#if PI_MACINTOSH_VERSION != 0
HFT gMacintoshHFT = 0;
ASUns32 gMacintoshVersion = 0;
#endif

#if PI_UNIX_VERSION != 0
HFT gUnixHFT = 0;
ASUns32 gUnixVersion = 0;
#endif

#if PI_WIN_VERSION != 0
HFT gWinHFT = 0;
ASUns32 gWinVersion = 0;
#endif

#if PI_ASEXTRA_VERSION != 0
HFT gASExtraHFT = 0;
ASUns32 gASExtraVersion = 0;
#endif

#if PI_ACROVIEW_SWEETPEA_VERSION != 0
HFT gAcroViewSweetPeaHFT = 0;
ASUns32 gAcroViewSweetPeaVersion = 0;
#endif

#if PI_PDMETADATA_VERSION != 0
HFT gPDMetadataHFT = 0;
ASUns32 gPDMetadataVersion = 0;
#endif

#define NEWEST_CORE_VERSION				 ((4L<<16) + 0)
#define NEWEST_ACROSUPPORT_VERSION		 ((5L<<16) + 0)
#define NEWEST_COS_VERSION				 ((5L<<16) + 0)
#define NEWEST_PDMODEL_VERSION			 ((5L<<16) + 0)
#define NEWEST_ACROVIEW_VERSION			 ((5L<<16) + 0)
#define NEWEST_MACINTOSH_VERSION		 ((2L<<16) + 2)
#define NEWEST_UNIX_VERSION				 ((2L<<16) + 0)
#define NEWEST_WIN_VERSION				 ((5L<<16) + 0)
#define NEWEST_PDFEDIT_WRITE_VERSION	 ((5L<<16) + 0)
#define NEWEST_PDFEDIT_READ_VERSION		 ((5L<<16) + 0)
#define NEWEST_PDSYSFONT_VERSION		 ((4L<<16) + 0)
#define NEWEST_PAGE_PDE_CONTENT_VERSION  ((4L<<16) + 0)
#define NEWEST_PDSEDIT_WRITE_VERSION	 ((5L<<16) + 0)
#define NEWEST_PDSEDIT_READ_VERSION		 ((5L<<16) + 0)
#define NEWEST_ASEXTRA_VERSION			 ((5L<<16) + 0)
#define NEWEST_ACROVIEW_SWEETPEA_VERSION ((5L<<16) + 0)
#define NEWEST_PDMETADATA_VERSION        ((5L<<16) + 0)

ExtensionID gExtensionID;		/* A identifying cookie sometimes needed by the application */

#if WIN_PLATFORM

#include <windows.h>

HINSTANCE 	gHINSTANCE;
HWND 		gHWND;

#if (!_USRDLL && !_AFXDLL)		/* MFC plugins have there own LibMain and _WEP */

/***************************************************************************************************
The DllMain Function

Unlike Windows 3.x DLLs, Windows NT calls one function, DllMain, for both initialization and
termination. It also makes calls on both a per-process and per-thread basis, so several initialization
calls can be made if a process is multithreaded.

DllMain uses the WINAPI convention and three parameters. The function returns
TRUE (1) to indicate success. If, during per-process initializati


The lpReserved parameter is reserved for the system's use and should not
be manipulated .

Win32 DLL initialization functions are passed the following information:

	The hModule parameter, a module handle.
	The ul_reason_for_call parameter, an enumerated type that indicates which
	of four reasons the LibMain procedure is being called:
		process attach,
		thread attach,
		thread detach,
		or process detach.

	The lpReserved parameter, which is unused.

	All calls to local memory management functions operate on the default heap.
	The command line can be obtained from GetCommandLine API function.
********************************************************************/


BOOL APIENTRY DllMain( HANDLE hModule,
                        DWORD ul_reason_for_call,
                        LPVOID lpReserved )
{
    switch( ul_reason_for_call ) {

    case DLL_PROCESS_ATTACH: //A new process is attempting to access the DLL; one thread is assumed.

	break;
    case DLL_THREAD_ATTACH://A new thread of an existing process is attempting to access the DLL; this call is made beginning with the second thread of a process attaching to the DLL.
    	break;
    case DLL_THREAD_DETACH://One of the additional threads (not the first thread) of a process is detaching from the DLL.
	break;

    case DLL_PROCESS_DETACH://A process is detaching from the DLL.
    	break;
    }
    return TRUE;
}

#endif /* MFC plugins */

/* This struct contains platform specific data useful to plug-ins.  This structure may grow over time
** to include new things, but, we will always support the old structure and only add to the end.
*/

typedef struct V0200_DATA_t_ {
	HWND 		hWnd;
	HINSTANCE	hInstance;
} V0200_DATA;

ASBool CALLBACK PlugInMain(ASInt32 appHandshakeVersion, ASInt32 *handshakeVersion, PISetupSDKProcType* setupProc, void* windowsData)

#else /* WIN_PLATFORM */

ACCB1 ASBool ACCB2 main(ASInt32 appHandshakeVersion, ASInt32 *handshakeVersion, PISetupSDKProcType *setupProc)

#endif /* else WIN_PLATFORM */
{

#if WIN_PLATFORM
	V0200_DATA* dataPtr = (V0200_DATA*) windowsData;
	gHWND = dataPtr->hWnd;
	gHINSTANCE = dataPtr->hInstance;
#endif /* WIN_PLATFORM */

#if MAC_PLATFORM
	gResFile = CurResFile();	// The plug in file.  Also contains default english resources
	gLocaleResFile = 0;			/* The localized resource file (if any). Can't set it here since I don't have the AV HFT yet. */
#endif /* MAC_PLATFORM */

	/*
	** appsHandshakeVersion tells us which version of the handshake struct the application has sent us.
	** HANDSHAKE_VERSION is the latest version that we, the plug-in, know about (see PIVersn.h)
	** Always use the earlier of the two structs to assure compatibility.
	** The version we want to use is returned to the application so it can adjust accordingly.
	*/
	*handshakeVersion = (appHandshakeVersion < HANDSHAKE_VERSION) ? appHandshakeVersion : HANDSHAKE_VERSION;

	/* Provide the routine for the host app to call to setup this plug-in */
	*setupProc = PISetupSDK;

	return true;
}

ACCB1 void ACCB2 RestorePlugInFrame(void *asEnviron)
{
	ACROlongjmp(*(ACROjmp_buf *)asEnviron, 1);

#if MAC_PLATFORM
	#if DEBUG
	DebugStr("\pRestorePluginFrame didn't jump anywhere!");
	#endif
#endif
}

/*
** This routine is called by the host application to set up the plug-in's SDK-provided functionality.
*/
ACCB1 ASBool ACCB2 PISetupSDK (ASUns32 handshakeVersion, void *sdkData)
{

	if (handshakeVersion == HANDSHAKE_V0200)
	{
		/* Cast sdkData to the appropriate type */
		PISDKData_V0200 *data = (PISDKData_V0200 *)sdkData;

		/* Belt and suspenders sanity check */
		if (data->handshakeVersion != HANDSHAKE_V0200)
			/* Someone lied */
			return false;

		/* Get our globals out */
		gExtensionID = data->extensionID;
		gCoreHFT = data->coreHFT;
		gCoreVersion = 0x00020000;

		/*
		** Note that we just got the Core HFT, so now we can, and are expected to, ASCallbackCreate()
		** every function we pass back to the viewer.
		** We can now also call functions in the Core HFT, such as ASExtensionMgrGetHFT().
		*/

		/* Get the HFTs we want */
#if PI_ACROSUPPORT_VERSION != 0
		gAcroSupportVersion = NEWEST_ACROSUPPORT_VERSION;
		while (gAcroSupportVersion >= PI_ACROSUPPORT_VERSION)
		{
			gAcroSupportHFT = ASExtensionMgrGetHFT(ASAtomFromString("AcroSupport"), gAcroSupportVersion);
			if (gAcroSupportHFT)
				break;
			gAcroSupportVersion--;
		}
		if (gAcroSupportVersion < PI_ACROSUPPORT_VERSION)
			return false;
#endif

#if PI_COS_VERSION != 0
		gCosVersion = NEWEST_COS_VERSION;
		while (gCosVersion >= PI_COS_VERSION)
		{
			gCosHFT = ASExtensionMgrGetHFT(ASAtomFromString("Cos"), gCosVersion);
			if (gCosHFT)
				break;
			gCosVersion--;
		}
		if (gCosVersion < PI_COS_VERSION)
			return false;
#endif

#if PI_PDMODEL_VERSION != 0
		gPDModelVersion = NEWEST_PDMODEL_VERSION;
		while (gPDModelVersion >= PI_PDMODEL_VERSION)
		{
			gPDModelHFT = ASExtensionMgrGetHFT(ASAtomFromString("PDModel"), gPDModelVersion);
			if (gPDModelHFT)
				break;
			gPDModelVersion--;
		}
		if (gPDModelVersion < PI_PDMODEL_VERSION)
			return false;
#endif


#if PI_PDFEDIT_WRITE_VERSION != 0
		gPDFEditWriteVersion = NEWEST_PDFEDIT_WRITE_VERSION;
		while (gPDFEditWriteVersion >= PI_PDFEDIT_WRITE_VERSION)
		{
			gPDFEditWriteHFT = ASExtensionMgrGetHFT(ASAtomFromString("PDFEditWrite"), gPDFEditWriteVersion);
			if (gPDFEditWriteHFT)
				break;
			gPDFEditWriteVersion--;
		}
		if (gPDFEditWriteVersion < PI_PDFEDIT_WRITE_VERSION)
			return false;
#endif

#if PI_PDFEDIT_READ_VERSION != 0
		gPDFEditReadVersion = NEWEST_PDFEDIT_READ_VERSION;
		while (gPDFEditReadVersion >= PI_PDFEDIT_READ_VERSION)
		{
			gPDFEditReadHFT = ASExtensionMgrGetHFT(ASAtomFromString("PDFEditRead"), gPDFEditReadVersion);
			if (gPDFEditReadHFT)
				break;
			gPDFEditReadVersion--;
		}
		if (gPDFEditReadVersion < PI_PDFEDIT_READ_VERSION)
			return false;
#endif

#if PI_PDSYSFONT_VERSION != 0
		gPDSysFontVersion = NEWEST_PDSYSFONT_VERSION;
		while (gPDSysFontVersion >= PI_PDSYSFONT_VERSION)
		{
			gPDSysFontHFT = ASExtensionMgrGetHFT(ASAtomFromString("PDSysFont"), gPDSysFontVersion);
			if (gPDSysFontHFT)
				break;
			gPDSysFontVersion--;
		}
		if (gPDSysFontVersion < PI_PDSYSFONT_VERSION)
			return false;
#endif

#if PI_PAGE_PDE_CONTENT_VERSION != 0
		gPagePDEContentVersion = NEWEST_PAGE_PDE_CONTENT_VERSION;
		while (gPagePDEContentVersion >= PI_PAGE_PDE_CONTENT_VERSION)
		{
			gPagePDEContentHFT = ASExtensionMgrGetHFT(ASAtomFromString("PagePDEContent"), gPagePDEContentVersion);
			if (gPagePDEContentHFT)
				break;
			gPagePDEContentVersion--;
		}
		if (gPagePDEContentVersion < PI_PAGE_PDE_CONTENT_VERSION)
			return false;
#endif

#if PI_ACROVIEW_VERSION != 0
		gAcroViewVersion = NEWEST_ACROVIEW_VERSION;
		while (gAcroViewVersion >= PI_ACROVIEW_VERSION)
		{
			gAcroViewHFT = ASExtensionMgrGetHFT(ASAtomFromString("AcroView"), gAcroViewVersion);
			if (gAcroViewHFT)
				break;
			gAcroViewVersion--;
		}
		if (gAcroViewVersion < PI_ACROVIEW_VERSION)
			return false;
#endif

#if PI_PDSEDIT_WRITE_VERSION != 0
		gPDSWriteVersion = NEWEST_PDSEDIT_WRITE_VERSION;
		while (gPDSWriteVersion >= PI_PDSEDIT_WRITE_VERSION)
		{
			gPDSWriteHFT = ASExtensionMgrGetHFT(ASAtomFromString("PDSWrite"), gPDSWriteVersion);
			if (gPDSWriteHFT)
				break;
			gPDSWriteVersion--;
		}
		if (gPDSWriteVersion < PI_PDSEDIT_WRITE_VERSION)
			return false;
#endif

#if PI_PDSEDIT_READ_VERSION != 0
		gPDSReadVersion = NEWEST_PDSEDIT_READ_VERSION;
		while (gPDSReadVersion >= PI_PDSEDIT_READ_VERSION)
		{
			gPDSReadHFT = ASExtensionMgrGetHFT(ASAtomFromString("PDSRead"), gPDSReadVersion);
			if (gPDSReadHFT)
				break;
			gPDSReadVersion--;
		}
		if (gPDSReadVersion < PI_PDSEDIT_READ_VERSION)
			return false;
#endif

#if PI_MACINTOSH_VERSION != 0
		gMacintoshVersion = NEWEST_MACINTOSH_VERSION;
		while (gMacintoshVersion >= PI_MACINTOSH_VERSION)
		{
			gMacintoshHFT = ASExtensionMgrGetHFT(ASAtomFromString("Macintosh"), gMacintoshVersion);
			if (gMacintoshHFT)
				break;
			gMacintoshVersion--;
		}
		if (gMacintoshVersion < PI_MACINTOSH_VERSION)
			return false;
#endif

#if PI_UNIX_VERSION != 0
		gUnixVersion = NEWEST_UNIX_VERSION;
		while (gUnixVersion >= PI_UNIX_VERSION)
		{
			gUnixHFT = ASExtensionMgrGetHFT(ASAtomFromString("Unix"), gUnixVersion);
			if (gUnixHFT)
				break;
			gUnixVersion--;
		}
		if (gUnixVersion < PI_UNIX_VERSION)
			return false;
#endif

#if PI_WIN_VERSION != 0
		gWinVersion = NEWEST_WIN_VERSION;
		while (gWinVersion >= PI_WIN_VERSION)
		{
			gWinHFT = ASExtensionMgrGetHFT(ASAtomFromString("Win"), gWinVersion);
			if (gWinHFT)
				break;
			gWinVersion--;
		}
		if (gWinVersion < PI_WIN_VERSION)
			return false;
#endif

#if PI_ASEXTRA_VERSION != 0
		gASExtraVersion = NEWEST_ASEXTRA_VERSION;
		while (gASExtraVersion >= PI_ASEXTRA_VERSION)
		{
			gASExtraHFT = ASExtensionMgrGetHFT(ASAtomFromString("ASExtra"), gASExtraVersion);
			if (gASExtraHFT)
				break;
			gASExtraVersion--;
		}
		if (gASExtraVersion < PI_ASEXTRA_VERSION)
			return false;
#endif

#ifndef UNIX_PLATFORM /* No ADM support on Unix */
#if PI_ACROVIEW_SWEETPEA_VERSION != 0
		gAcroViewSweetPeaVersion = NEWEST_ACROVIEW_SWEETPEA_VERSION;
		while (gAcroViewSweetPeaVersion >= PI_ACROVIEW_SWEETPEA_VERSION)
		{
			gAcroViewSweetPeaHFT = ASExtensionMgrGetHFT(ASAtomFromString("AcroViewSweetPea"), gAcroViewSweetPeaVersion);
			if (gAcroViewSweetPeaHFT)
				break;
			gAcroViewSweetPeaVersion--;
		}
		if (gAcroViewSweetPeaVersion < PI_ACROVIEW_SWEETPEA_VERSION)
			return false;
#endif
#endif

#if PI_PDMETADATA_VERSION != 0
		gPDMetadataVersion = NEWEST_PDMETADATA_VERSION;
		while (gPDMetadataVersion >= PI_PDMETADATA_VERSION)
		{
			gPDMetadataHFT = ASExtensionMgrGetHFT(ASAtomFromString("PDMetadata"), gPDMetadataVersion);
			if (gPDMetadataHFT)
				break;
			gPDMetadataVersion--;
		}
		if (gPDMetadataVersion < PI_PDMETADATA_VERSION)
			return false;
#endif
		/* Set the plug-in's handshake routine, which is called next by the host application */
		data->handshakeCallback = ASCallbackCreateProto(PIHandshakeProcType, PIHandshake);

		/* Return success */
		return true;

	} /* Each time the handshake version changes, add a new "else if" branch here */

	/*
	** If we reach here, then we were passed a handshake version number we don't know about.
	** This shouldn't ever happen since our main() routine chose the version number.
	*/
	return false;
}

