// -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// 
// This is part of JumpSection: www.pdfhacks.com/jumpsection/
// Copyright (c) 2004 O'Reilly and Associates
// Written by Sid Steward
//

#include "windows.h"

#include "PIMain.5.h"

#include "ASCalls.h"  // includes PIRequir.h; ASExpT.h; CorCalls.h
#include "AVCalls.h"  // includes PIRequir.h; AVExpT.h; NSelExpT.h; CorCalls.h
#include "PDCalls.h"  // includes PIRequir.h; PDExpT.h; 

#include <string>
using namespace std;

static const char g_PluginName[]= "BBKS:AccessPDF:JumpSection";
static const char g_PluginTitle[]= "JumpSection";

static ACCB1 ASBool ACCB2 
PluginInit(void);
//
// our plug-in's bootstrap

static ACCB1 ASBool ACCB2 
PluginUnInit(void);
//
// clean up after ourselves

static void 
TestExceptionHandling() { ASRaise( ErrSysPDDoc ); }
//
// throw an exception just to see how our handling takes it


////
// PIHandshake is declared in PICommon.h, which is included
// by PIMain.h; this is how you tell Acrobat which functions
// to call for initialization, etc.
// 
extern "C" {

  ACCB1 ASBool ACCB2 
  PIHandshake(ASUns32 handshakeVersion, void *handshakeData)
  {
    if( handshakeVersion == HANDSHAKE_V0200 ) {
      PIHandshakeData_V0200 *hsData= 
        (PIHandshakeData_V0200 *)handshakeData;

      hsData->extensionName= 
        ASAtomFromString( g_PluginName );
      hsData->exportHFTsCallback= 0;
      hsData->importReplaceAndRegisterCallback= 0;
      hsData->initCallback= 
        (void*)ASCallbackCreateProto( PIInitProcType, (void*)(PluginInit) );
      hsData->unloadCallback= 
        (void*)ASCallbackCreateProto( PIUnloadProcType, (void*)(PluginUnInit) );

      return true;
  
    } // Each time the handshake version changes, add a new "else if" branch

    return false;
  }

}

// find the bookmarks closest to the given page number; we make a point
// of going over all document bookmarks, since the bookmark order might
// not be the same as the document reading order
//
static bool
FindBoundingBookmarks
( AVDoc avDoc,
  PDDoc pdDoc,
  PDBookmark parent,
  PDBookmark& prev_bookmark,
  ASInt32& prev_bookmark_page_num,
  PDBookmark& next_bookmark,
  ASInt32& next_bookmark_page_num,
  ASInt32 curr_page_num )
{
  bool continue_b= true;
  PDBookmark child= PDBookmarkGetFirstChild( parent );

  while( PDBookmarkIsValid( child ) ) {
    PDAction action= PDBookmarkGetAction( child );
    ASInt32 page_num= 0; {
      PDViewDestination dest= PDActionGetDest( action );
      dest= PDViewDestResolve( dest, pdDoc );
      
      ASAtom fit;
      ASFixedRect destRect;
      ASFixed zoom;

      PDViewDestGetAttr( dest, &page_num, &fit, &destRect, &zoom );
    }

    if( page_num< curr_page_num &&
        ( prev_bookmark_page_num== -1 || 
          prev_bookmark_page_num< page_num ) )
      {
        prev_bookmark= child;
        prev_bookmark_page_num= page_num;
      }
    if( curr_page_num< page_num &&
        ( next_bookmark_page_num== -1 || 
          page_num< next_bookmark_page_num ) )
      {
        next_bookmark= child;
        next_bookmark_page_num= page_num;
      }

    // recurse
    continue_b= 
      FindBoundingBookmarks
      ( avDoc,
        pdDoc,
        child, 
        prev_bookmark,
        prev_bookmark_page_num,
        next_bookmark,
        next_bookmark_page_num,
        curr_page_num );
      
    // get next sibling
    child= PDBookmarkGetNext( child );
  }

  return continue_b;
}

ACCB1 ASBool ACCB2
KeyDownProc( AVPageView page_view, 
             ASUns16 code, 
             ASInt16 modifier,
             void* data )
{
  if( code== 28 && modifier== 1 || // SHIFT-left-arrow
      code== 29 && modifier== 1 ) // SHIFT-right-arrow
    {
      AVDoc avDoc= AVPageViewGetAVDoc( page_view );
      if( avDoc ) {
        PDDoc pdDoc= AVDocGetPDDoc( avDoc );
        PDBookmark bookmark_root= PDDocGetBookmarkRoot( pdDoc );
        PDBookmark prev_bookmark= bookmark_root;
        ASInt32 prev_bookmark_page_num= -1;
        PDBookmark next_bookmark= bookmark_root;
        ASInt32 next_bookmark_page_num= -1;

        FindBoundingBookmarks
          ( avDoc,
            pdDoc,
            bookmark_root,
            prev_bookmark,
            prev_bookmark_page_num,
            next_bookmark,
            next_bookmark_page_num,
            AVPageViewGetPageNum( page_view ) );

        if( code== 28 && modifier== 1 && prev_bookmark_page_num!= -1 ) {
          // SHIFT-left-arrow
          AVDocPerformAction( avDoc, PDBookmarkGetAction(prev_bookmark) );
          return true;
        }
        else if( code== 29 && modifier== 1 && next_bookmark_page_num!= -1 ) {
          // SHIFT-right-arrow
          AVDocPerformAction( avDoc, PDBookmarkGetAction(next_bookmark) );
          return true;
        }
      }
    }

  return false;
}

ASCallback g_KeyDownCallback= 0;

static ACCB1 ASBool ACCB2 
PluginInit( void )
{
  AVAppRegisterForPageViewKeyDown( KeyDownProc, 0 );

  return true;
}

static ACCB1 ASBool ACCB2 
PluginUnInit(void)
{
  AVAppUnregisterForPageViewKeyDown( KeyDownProc );

  return true;
}

static ACCB1 ASBool ACCB2
PluginImportHFT(void)
{
  return true;
}
